@@ -98,7 +98,13 @@ extern "C" {
 #  define MRN_TABLE_LIST_INIT_REQUIRE_ALIAS
 #endif
 
-#ifdef BIG_TABLES
+#if MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)
+#  define MRN_BIG_TABLES
+#elif defined(BIG_TABLES)
+#  define MRN_BIG_TABLES
+#endif
+
+#ifdef MRN_BIG_TABLES
 #  define MRN_HA_ROWS_FORMAT "llu"
 #else
 #  define MRN_HA_ROWS_FORMAT "lu"
@@ -145,6 +151,14 @@ extern "C" {
 #  define MRN_HAVE_TL_WRITE_ALLOW_READ
 #endif
 
+#if MYSQL_VERSION_ID < 50706 || defined(MRN_MARIADB_P)
+#  define MRN_HAVE_TL_WRITE_DELAYED
+#endif
+
+#if MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)
+#  define MRN_HAVE_TL_WRITE_CONCURRENT_DEFAULT
+#endif
+
 #if (defined(MRN_MARIADB_P) && \
      ((MYSQL_VERSION_ID >= 50306 && MYSQL_VERSION_ID < 50500) || \
       MYSQL_VERSION_ID >= 50523))
@@ -179,6 +193,44 @@ extern "C" {
 #  define MRN_HAVE_TDC_LOCK_TABLE_SHARE
 #endif
 
+#ifdef MRN_MARIADB_P
+#  if MYSQL_VERSION_ID >= 50542 && MYSQL_VERSION_ID < 100000
+#    define MRN_SUPPORT_THDVAR_SET
+#  elif MYSQL_VERSION_ID >= 100017
+#    define MRN_SUPPORT_THDVAR_SET
+#  endif
+#else
+#  define MRN_SUPPORT_THDVAR_SET
+#endif
+
+#ifdef MRN_MARIADB_P
+#  if MYSQL_VERSION_ID < 100000
+#    define MRN_SUPPORT_PARTITION
+#  endif
+#else
+#  define MRN_SUPPORT_PARTITION
+#endif
+
+#if MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)
+#  define MRN_FLUSH_LOGS_HAVE_BINLOG_GROUP_FLUSH
+#endif
+
+#if MYSQL_VERSION_ID < 50706 || defined(MRN_MARIADB_P)
+#  define MRN_HAVE_HTON_ALTER_TABLE_FLAGS
+#endif
+
+#if MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)
+#  define MRN_FOREIGN_KEY_USE_CONST_STRING
+#endif
+
+#if MYSQL_VERSION_ID < 50706 || defined(MRN_MARIADB_P)
+#  define MRN_HANDLER_IS_FATAL_ERROR_HAVE_FLAGS
+#endif
+
+#if MYSQL_VERSION_ID < 50706 || defined(MRN_MARIADB_P)
+#  define MRN_HANDLER_HAVE_RESET_AUTO_INCREMENT
+#endif
+
 class ha_mroonga;
 
 /* structs */
@@ -348,7 +400,7 @@ public:
 
   uint max_supported_record_length()   const;
   uint max_supported_keys()            const;
-  uint max_supported_key_parts();
+  uint max_supported_key_parts()       const;
   uint max_supported_key_length()      const;
   uint max_supported_key_part_length() const;
 
@@ -448,7 +500,7 @@ public:
   bool check_and_repair(THD *thd);
   int analyze(THD* thd, HA_CHECK_OPT* check_opt);
   int optimize(THD* thd, HA_CHECK_OPT* check_opt);
-  bool is_fatal_error(int error_num, uint flags);
+  bool is_fatal_error(int error_num, uint flags=0);
   bool check_if_incompatible_data(HA_CREATE_INFO *create_info,
                                   uint table_changes);
 #ifdef MRN_HANDLER_HAVE_CHECK_IF_SUPPORTED_INPLACE_ALTER
@@ -474,7 +526,9 @@ public:
   void restore_auto_increment(ulonglong prev_insert_id);
   void release_auto_increment();
   int check_for_upgrade(HA_CHECK_OPT *check_opt);
+#ifdef MRN_HANDLER_HAVE_RESET_AUTO_INCREMENT
   int reset_auto_increment(ulonglong value);
+#endif
   bool was_semi_consistent_read();
   void try_semi_consistent_read(bool yes);
   void unlock_row();
@@ -556,6 +610,7 @@ private:
   int drop_index(MRN_SHARE *target_share, uint key_index);
   grn_obj *find_tokenizer(const char *name, int name_length);
   grn_obj *find_normalizer(KEY *key_info);
+  bool find_index_column_flags(KEY *key_info, grn_obj_flags *index_column_flags);
   bool find_token_filters(KEY *key_info, grn_obj *token_filters);
   bool find_token_filters_put(grn_obj *token_filters,
                               const char *token_filter_name,
@@ -812,8 +867,8 @@ private:
   uint storage_max_supported_record_length() const;
   uint wrapper_max_supported_keys() const;
   uint storage_max_supported_keys() const;
-  uint wrapper_max_supported_key_parts();
-  uint storage_max_supported_key_parts();
+  uint wrapper_max_supported_key_parts() const;
+  uint storage_max_supported_key_parts() const;
   uint wrapper_max_supported_key_length() const;
   uint storage_max_supported_key_length() const;
   uint wrapper_max_supported_key_part_length() const;
@@ -1131,8 +1186,10 @@ private:
   void storage_release_auto_increment();
   int wrapper_check_for_upgrade(HA_CHECK_OPT *check_opt);
   int storage_check_for_upgrade(HA_CHECK_OPT *check_opt);
+#ifdef MRN_HANDLER_HAVE_RESET_AUTO_INCREMENT
   int wrapper_reset_auto_increment(ulonglong value);
   int storage_reset_auto_increment(ulonglong value);
+#endif
   bool wrapper_was_semi_consistent_read();
   bool storage_was_semi_consistent_read();
   void wrapper_try_semi_consistent_read(bool yes);
